@@ -49,7 +49,7 @@ class TestDeploy(testtools.TestCase):
                                 candidates=None)
         reserve_defaults.update(reserve_args)
 
-        provision_defaults = dict(image='myimg',
+        provision_defaults = dict(image=mock.ANY,
                                   nics=[{'network': 'mynet'}],
                                   root_size_gb=None,
                                   swap_size_mb=None,
@@ -75,7 +75,7 @@ class TestDeploy(testtools.TestCase):
         instance = mock_pr.return_value.provision_node.return_value
         instance.create_autospec(_instance.Instance)
         instance.node.name = None
-        instance.node.uuid = '123'
+        instance.node.id = '123'
         instance.state = 'active'
         instance.is_deployed = True
         instance.ip_addresses.return_value = {'private': ['1.2.3.4']}
@@ -88,6 +88,10 @@ class TestDeploy(testtools.TestCase):
         self.assertEqual([], config.ssh_keys)
         mock_log.basicConfig.assert_called_once_with(level=mock_log.WARNING,
                                                      format=mock.ANY)
+
+        source = mock_pr.return_value.provision_node.call_args[1]['image']
+        self.assertIsInstance(source, sources.GlanceImage)
+        self.assertEqual("myimg", source.image)
         self.assertEqual(
             mock.call('metalsmith').setLevel(mock_log.WARNING).call_list() +
             mock.call(_cmd._URLLIB3_LOGGER).setLevel(
@@ -127,7 +131,7 @@ class TestDeploy(testtools.TestCase):
         instance.is_deployed = True
         instance.ip_addresses.return_value = {}
         instance.node.name = None
-        instance.node.uuid = '123'
+        instance.node.id = '123'
         instance.state = 'active'
 
         args = ['deploy', '--network', 'mynet', '--image', 'myimg',
@@ -142,7 +146,7 @@ class TestDeploy(testtools.TestCase):
         instance.create_autospec(_instance.Instance)
         instance.is_deployed = False
         instance.node.name = None
-        instance.node.uuid = '123'
+        instance.node.id = '123'
         instance.state = 'deploying'
 
         args = ['deploy', '--network', 'mynet', '--image', 'myimg',
@@ -338,6 +342,11 @@ class TestDeploy(testtools.TestCase):
                     {'nics': [{'network': 'private', 'fixed_ip': '10.0.0.2'},
                               {'network': 'public', 'fixed_ip': '8.0.8.0'}]})
 
+    def test_args_subnet(self, mock_pr):
+        args = ['deploy', '--subnet', 'mysubnet', '--image', 'myimg',
+                '--resource-class', 'compute']
+        self._check(mock_pr, args, {}, {'nics': [{'subnet': 'mysubnet'}]})
+
     def test_args_bad_ip(self, mock_pr):
         args = ['deploy', '--image', 'myimg', '--resource-class', 'compute',
                 '--ip', 'private:10.0.0.2', '--ip', 'public']
@@ -487,7 +496,7 @@ class TestUndeploy(testtools.TestCase):
 
     def test_ok(self, mock_os_conf, mock_pr):
         node = mock_pr.return_value.unprovision_node.return_value
-        node.uuid = '123'
+        node.id = '123'
         node.name = None
         node.provision_state = 'cleaning'
 
@@ -506,7 +515,7 @@ class TestUndeploy(testtools.TestCase):
 
     def test_custom_wait(self, mock_os_conf, mock_pr):
         node = mock_pr.return_value.unprovision_node.return_value
-        node.uuid = '123'
+        node.id = '123'
         node.name = None
         node.provision_state = 'available'
 
@@ -580,9 +589,9 @@ class TestShowWait(testtools.TestCase):
             for hostname in ['hostname1', 'hostname2']
         ]
         for inst in self.instances:
-            inst.node.uuid = inst.uuid
+            inst.node.id = inst.uuid
             inst.node.name = 'name-%s' % inst.uuid
-            inst.to_dict.return_value = {inst.node.uuid: inst.node.name}
+            inst.to_dict.return_value = {inst.node.id: inst.node.name}
 
     def test_show(self, mock_os_conf, mock_pr):
         mock_pr.return_value.show_instances.return_value = self.instances
